/* ROSA Media Player
 * Copyright (c) 2012 ROSA  <support@rosalab.ru>
 * Authors:
 *  Julia Mineeva <julia.mineeva@osinit.ru>
 *  Sergey Borovkov <sergey.borovkov@osinit.ru>
 * License: GPLv3
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "request.h"

#include <QtNetwork/QNetworkAccessManager>
#ifdef Q_OS_WIN
#include <QJson/Parser>
#else
#include <qjson/parser.h>
#endif

QNetworkAccessManager *Request::manager = 0;

Request::Request(QObject *parent)
    : QObject(parent)
{
    // this is never deleted
    if (!manager)
        manager = new QNetworkAccessManager();
}

void Request::setUrl(const QUrl &url)
{
    m_url = url;
}

void Request::start()
{
    if (m_url.isEmpty())
        return;

    QNetworkRequest request(m_url);

    QNetworkReply *reply  = manager->get(request);
    connect(reply, SIGNAL(finished()), SLOT(replyFinished()));
    connect(reply, SIGNAL(finished()), SIGNAL(success()));
    connect(reply, SIGNAL(error(QNetworkReply::NetworkError)), SLOT(gotError(QNetworkReply::NetworkError)));
}

void Request::replyFinished()
{
    QNetworkReply *reply = qobject_cast<QNetworkReply *>(sender());
    QByteArray answer = reply->readAll();
    emit replyReady(answer);
    reply->deleteLater();
}

void Request::gotError(QNetworkReply::NetworkError err)
{
    Q_UNUSED(err)
    QNetworkReply *reply = qobject_cast<QNetworkReply *>(sender());
    emit error(reply->errorString());
    reply->deleteLater();
}
